local MAJOR_VERSION = "Threat-1.0"
local MINOR_VERSION = tonumber(("$Revision: 39751 $"):match("%d+"))

if MINOR_VERSION > _G.ThreatLib_MINOR_VERSION then
	_G.ThreatLib_MINOR_VERSION = MINOR_VERSION
end

local _, c = _G.UnitClass("player")
if c ~= "HUNTER" then return end

ThreatLib_funcs[#ThreatLib_funcs+1] = function()

local _G = _G
local tonumber = _G.tonumber

local AceLibrary = _G.AceLibrary

local ThreatLib = _G.ThreatLib

local BS = AceLibrary("Babble-Spell-2.2")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local Hunter = ThreatLib:GetModule("ClassCore"):NewModule(c)

local distractThreatAmounts		= {110, 160, 250, 350, 465, 600, 900}
local disengageThreatAmounts	= {-140, -280, -405, -545}
	
function Hunter:ClassInit()
	self.className = "HUNTER"
	
	-- CastHandlers
	self.CastHandlers[BS["Distracting Shot"]] 	= self.DistractingShot
	self.CastHandlers[BS["Disengage"]]			= self.Disengage
	
	self.CastHandlers[BS["Feign Death"]]		= self.FeignDeath
	self.CastMissHandlers[BS["Feign Death"]]	= self.FeignDeathMiss

	-- ClassBuffs
	self.ClassBuffs[BS["Misdirection"]]			= self.MisdirectionBuff
end

function Hunter:DistractingShot(rank)
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(distractThreatAmounts[ranknum] * self:threatMods())
end

function Hunter:Disengage(rank)
	local ranknum = tonumber(rank:match("%d+"))
	self:AddTargetThreat(disengageThreatAmounts[ranknum] * self:threatMods())
end

local function applyFeignDeath(self, target)
	self:_multiplyTargetThreat(0, target)
end

local function applyFeignDeathGlobal(self)
	self:_multiplyThreat(0)
end

function Hunter:FeignDeath()
	-- for each of our targets schedule a clear in a half second that will get cancelled if we get individual resists
	for target in pairs(self.targetThreat) do
		self:ScheduleEvent("ThreatLib-Hunter-Feign-" .. target, applyFeignDeath, 0.75, self, target)
	end
	self:ScheduleEvent("ThreatLib-Hunter-Feign-GlobalThreat", applyFeignDeathGlobal, 0.75, self)
end

function Hunter:FeignDeathMiss(target)
	if self:IsEventScheduled("ThreatLib-Hunter-Feign-" .. target) then
		self:CancelScheduledEvent("ThreatLib-Hunter-Feign-" .. target)
	end
end

function Hunter:MisdirectionBuff(action, rank, apps)
	-- Previous implementation was a bit too clever :P
	if action == "lose" then
		self:RedirectThreatTo(nil)
	elseif action == "gain" then
		self:RedirectThreatTo(self.currentTarget)
	end
end

table.insert(ThreatLib.UpvalueFixers, function(lib)
	ThreatLib = lib
	Hunter = ThreatLib:GetModule("ClassCore"):GetModule(c)
end)

end
